/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv3901.h - Declaration file for utility functions and global variables
 *  DRV3901-Q1EVM_FIRMWARE
 *  09/02/2022
 *  Murugavel Raju
 *
 ******************************************************************************/

#ifndef DRV3901_H_
#define DRV3901_H_

/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include "drv3901-q1/drv3901-q1-RegMap.h"
#include "msp430.h"

/******************************************************************************
 * MACROS
 *****************************************************************************/

// GPIO Port 1 Definitions
#define MCU_STATUS  BIT0           // P1.0: Status LED output
#define TP13        BIT1           // P1.1: Test Point TP13
#define TP14        BIT2           // P1.2: Test Point TP14
#define PUSH        BIT3           // P1.3: Push button
#define FAULT_A     BIT4           // P1.4: nFAULT for DRV A
#define FAULT_B     BIT5           // P1.5: nFAULT for DRV B
#define TRIG2       BIT6           // P1.6: TRG1 output
#define TP6         BIT7           // P1.7: TP6

// GPIO Port 2 Definitions
#define FAULT_A_LED BIT5           // P2.5: FAULT_A status LED output
#define FAULT_B_LED BIT6           // P2.6: FAULT_B status LED output
#define nSCS_B      BIT7           // P2.7: nSCS_B control output

// GPIO Port 3 Definitions
#define nSCS_A      BIT0           // P3.0: STE nSCS_A control output UCB0STE
#define SDI         BIT1           // P3.1: Serial data input UCB0SIMO
#define SDO         BIT2           // P3.2: Serial data output UCB0SOMI
#define SCLK        BIT3           // P3.3: Serial clock output UCB0CLK
#define TX          BIT4           // P3.4: UART TXD UCA0TXD
#define RX          BIT5           // P3.5: UART RXD UCA0RXD
#define GUI_STATUS  BIT6           // P3.6: GUI status LED output
#define TP33        BIT7           // P3.7: TP33

// GPIO Port 4 Definitions
#define DRV_A_LED   BIT0           // P4.0: DRV A status LED output
#define TRIG1       BIT1           // P4.1: TRG1 output
#define DRV_B_LED   BIT2           // P4.2: DRV B status LED output
#define TP17        BIT3           // P4.3: Test Point TP17
#define TP20        BIT4           // P4.4: Test Point TP20
#define TP21        BIT5           // P4.5: Test Point TP21
#define TP31        BIT6           // P4.6: Test Point TP31
#define TP32        BIT7           // P4.7: Test Point TP32

// gpio Macros
#define read_FAULT_A_pin       (P1IN & FAULT_A)       // Read FAULT_A status
#define read_FAULT_B_pin       (P1IN & FAULT_B)       // Read FAULT_B status

#define REV_1p0             1
#define REV_2p0             2

#define REG_READ_PERIOD    96                         // Register read period 96 x 4 ms = 384 ms

/******************************************************************************
* DATA STRUCTURES
 *****************************************************************************/
typedef struct DRV3901_Device_Obj
{
    // starting address = 0x0990, Number of bytes = 32 bytes, including one filler byte
    // for MSP430 even address alignment one after DRVB_FAULT location and another after ReadCmd location
    float firmwareVersion;                            // EVM firmware version

    // Device Parameters
    volatile uint16_t deviceID;                       // device ID value

    volatile uint8_t TRG1;                            // 0: OUT1 disabled, 1: OUT1 enabled
    volatile uint8_t TRG2;                            // 0: OUT2 disabled, 1: OUT2 enabled
    volatile uint8_t drv_SPI_mode;                    // 0: peer to peer SPI, 1: addressable SPI
    volatile uint8_t TRG1a;                           // 0 to 0xb 12 selections
    volatile uint8_t TRG2a;                           // 0 to 0xb 12 selections
    volatile uint8_t NAD_A;                           // variable for DRV A NAD, b00:5.6Kohm, b01:12Kohm,b10:27Kohm,b11:56Kohm
    volatile uint8_t NAD_B;                           // variable for DRV B NAD, b00:5.6Kohm, b01:12Kohm,b10:27Kohm,b11:56Kohm
    volatile uint8_t DRVA_SEL;                        // DRVA selected
    volatile uint8_t DRVB_SEL;                        // DRVB selected
    volatile uint8_t DRVA_FAULT;                      // 0: No Fault, 1: Fault
    volatile uint8_t DRVB_FAULT;                      // 0: No Fault, 1: Fault
    volatile uint8_t AutoRead;                        // 0: Auto Read off, 1: Auto Read on

    // Register Read Write
    uint16_t   ManWriteAddr;                          // Manual write Address variable
    uint16_t   ManReadAddr;                           // Manual read Address variable
    uint16_t   ManWriteData;                          // Manual write data variable
    uint16_t   ManReadData;                           // Manual read data variable
    bool       ManWriteCmd;                           // Manual Register write GUIcommand
    bool       ManReadCmd;                            // Manual Register read GUI command
    bool       ReadCmd;                               // Registers read GUI command
    // Silicon revision
    volatile uint16_t revID;                          // device rev ID
} DRV3901_Q1_Device_Obj_t;

/******************************************************************************
 * VARIABLES FOR UTILITY
 *****************************************************************************/
volatile uint8_t RegReadPeriod;
volatile uint8_t ReadCMD;
volatile uint8_t TRG1state;
volatile uint8_t TRG2state;

/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
extern volatile DRV3901_Q1_Device_Obj_t gBrushedObj;
extern volatile DRV3901_Q1_REG_t gDeviceSpiReg;
extern volatile uint8_t configx[10];                  // Array for CONFIGx upper and lower bytes

// Function Prototypes
void gui_deviceControlUpdateTask(void);
void gui_regUpdateTask(void);
void gui_regRead(void);
void mcu_init(void);
void getEVMID(void);
void drv3901q1evm_init(void);
inline void drv_RESET(void);
void drv3901q1_RegistersRead();
void TRIG1_pwm16k(uint8_t duty);
void TRIG1_pwm125k(uint8_t duty);
void TRIG2_pwm16k(uint8_t duty);
void TRIG2_pwm125k(uint8_t duty);

#endif // DRV3901_H_
