/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  SPI.c - Definition file for utility functions
 *  drv SPI FIRMWARE frame work adapted for drv3901-q1
 *  07/22/2022
 *  Murugavel Raju
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include "drv3901-q1/drv3901-q1.h"
#include "spi.h"
#include "crc.h"

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
// SPI Initialization
void spiInit(void)
{
    P3OUT |= nSCS_A;                                       // nSCS_A set to high
    P3DIR |= nSCS_A;                                       // nSCS_A gpio output

    P2OUT |= nSCS_B;                                       // nSCS_B set to high
    P2DIR |= nSCS_B;                                       // nSCS_B gpio output

    P3SEL |= (SDI | SDO | SCLK);                           // Enable SPI functions for gpio

    UCB0CTL1 = UCSWRST;
    UCB0CTL0 |= UCMSB | UCMST | UCSYNC;                    // MSB first SPI controller
    UCB0CTL1 |= UCSSEL_2;                                  // SMCLK 16MHz
    UCB0BR0 |= 16;                                         // 1MHz
    UCB0BR1 = 0;                                           //
    UCB0CTL1 &= ~UCSWRST;                                  // Initialize USCI state machine
} // end spiInit()

// SPI read register to read device configurations and operating parameters
// Register format SDI |A6|A5|A4|A3|A2|A1|A0|R/W|D15|D14|D13|D12|D11|D10|D9|D8|D7|D6|D5|D4|D3|D2|D1|D0|
// Register format SD0 |Hi-Z|Hi-z|A5|A4|A3|A2|A1|A0|D15|D14|D13|D12|D11|D10|D9|D8|D7|D6|D5|D4|D3|D2|D1|D0|
// Ax are address bits, Dx are data bits and R/W is read write bit R/W = 1 for read
uint16_t spi_readRegister(uint8_t address)
{
    volatile uint16_t reg_value = 0;
    volatile uint8_t reg_address = 0;
    volatile uint8_t dataMSB = 0;
    volatile uint8_t dataLSB = 0;

    reg_address |= SPI_RW_BIT_MASK;                        // Set R/W bit
    reg_address |= ((address << SPI_ADDRESS_POS) & SPI_ADDRESS_MASK); // Configure register address value

    if (!gBrushedObj.drv_SPI_mode)
    {
        if (gBrushedObj.DRVA_SEL)                          // Driver A selected
        {
            reg_address |= ((gBrushedObj.NAD_A << SPI_NAD_POS) & SPI_NAD_MASK); // Write NAD_A bits
            P3OUT &= ~nSCS_A;                              //SPI nSCS_A low
        }
        else if (gBrushedObj.DRVB_SEL)                     // Driver B selected
        {
            reg_address |= ((gBrushedObj.NAD_B << SPI_NAD_POS) & SPI_NAD_MASK); // Write NAD_B bits
            P2OUT &= ~nSCS_B;                              //SPI nSCS_A low
        }
    }
    else if (gBrushedObj.drv_SPI_mode)
    {
        if (gBrushedObj.DRVA_SEL)                          // Driver A selected
        {
            reg_address |= ((gBrushedObj.NAD_A << SPI_NAD_POS) & SPI_NAD_MASK); // Write NAD_A bits
            P3DIR &= ~nSCS_A;                              // nSCS_A set to HiZ
            P2OUT &= ~nSCS_B;                              //SPI nSCS_A low
        }
        else if (gBrushedObj.DRVB_SEL)                     // Driver B selected
        {
            reg_address |= ((gBrushedObj.NAD_B << SPI_NAD_POS) & SPI_NAD_MASK); // Write NAD_B bits
            P3DIR &= ~nSCS_A;                              // nSCS_A set to HiZ
            P2OUT &= ~nSCS_B;                              //SPI nSCS_A low
        }
    }

    while (!(IFG2 & UCB0TXIFG));                           // USCI_B0 TX buffer ready

    UCB0TXBUF = reg_address;                               // Transmit the Address(first byte)
    while (UCB0STAT & SPI_BUSY_FLAG);                      // Wait till Transmission is complete
    dataMSB = UCB0RXBUF;                                   // Receive the First byte of data, MSB byte

    UCB0TXBUF = 0x00;                                      // Transmit dummy data second byte
    while (UCB0STAT & SPI_BUSY_FLAG);                      // Wait till Transmission is complete
    dataMSB = UCB0RXBUF & 0xFF;                            // Receive the second byte,MSB byte.

    UCB0TXBUF = 0x00;                                      // Transmit dummy data third byte
    while (UCB0STAT & SPI_BUSY_FLAG);                      // Wait till Transmission is complete
    dataLSB = UCB0RXBUF & 0xFF;                            // Receive the third byte,LSB byte

    if (!gBrushedObj.drv_SPI_mode)
    {
        if (gBrushedObj.DRVA_SEL)                          // Driver A selected
        {
            P3OUT |= nSCS_A;                               // SPI nSCS_A high
        }
        else if (gBrushedObj.DRVB_SEL)                     // Driver B selected
        {
            P2OUT |= nSCS_B;                               // SPI nSCS_B high
        }
    }
    else if (gBrushedObj.drv_SPI_mode)
    {
        if (gBrushedObj.DRVA_SEL)                          // Driver A selected
        {
            P2OUT |= nSCS_B;                               // SPI nSCS_B high
            P3OUT |= nSCS_A;                               // SPI nSCS_A high
            P3DIR |= nSCS_A;                               // nSCS_A gpio output
        }
        else if (gBrushedObj.DRVB_SEL)                     // Driver B selected
        {
            P2OUT |= nSCS_B;                               // SPI nSCS_B high
        }
    }

    _delay_cycles(10);                                     // 625 ns settling delay
    reg_value = (((dataMSB << SPI_UPPER_DATA_POS) & SPI_UPPER_DATA_MASK) | ((dataLSB << SPI_LOWER_DATA_POS) & SPI_LOWER_DATA_MASK));  // Convert to 16-bit data
    return (reg_value);
} // end spi_readRegister()

// SPI write register to set device configurations and operating parameters
// Register format SDI |A6|A5|A4|A3|A2|A1|A0|R/W|D15|D14|D13|D12|D11|D10|D9|D8|D7|D6|D5|D4|D3|D2|D1|D0|
// Register format SD0 |Hi-Z|Hi-z|A5|A4|A3|A2|A1|A0|D15|D14|D13|D12|D11|D10|D9|D8|D7|D6|D5|D4|D3|D2|D1|D0|
// Ax are address bits, Dx are data bits and R/W is read write bit R/W = 0 for write
uint16_t spi_writeRegister(uint8_t address, uint16_t data)
{
    volatile uint16_t reg_value = 0;
    volatile uint8_t reg_address = 0;
    volatile uint8_t crc_cmd = 0;
    volatile uint8_t crc_config = 0;

    if (gBrushedObj.DRVA_SEL)                              // Driver A selected
        reg_address |= ((gBrushedObj.NAD_A << SPI_NAD_POS) & SPI_NAD_MASK); // Write NAD_A bits
    else if (gBrushedObj.DRVB_SEL)                         // Driver B selected
        reg_address |= ((gBrushedObj.NAD_B << SPI_NAD_POS) & SPI_NAD_MASK); // Write NAD_B bits

    reg_address |= ((address << SPI_ADDRESS_POS) & SPI_ADDRESS_MASK); // Write address value
    reg_address &= ~SPI_RW_BIT_MASK;                       // Clear R/W bit

    if((address == SPI_REG_CMD0) || (address == SPI_REG_CMD1) || (address == SPI_REG_CMD2)) // CMDx register selected
    {
        reg_value |= (data & SPI_UPPER_DATA_MASK);         // upper data byte
        crc_cmd = CRC_CMD(reg_address, data);
        reg_value |= (crc_cmd & SPI_LOWER_DATA_MASK);      // lower data byte is crc

        // write to selected device registers
        if (!gBrushedObj.drv_SPI_mode)
        {
            if (gBrushedObj.DRVA_SEL)                      // Driver A selected
            {
                P3OUT &= ~nSCS_A;                          // SPI nSCS_A low
            }
            else if (gBrushedObj.DRVB_SEL)                 // Driver B selected
            {
                P2OUT &= ~nSCS_B;                          // SPI nSCS_B low
            }
        }
        else if (gBrushedObj.drv_SPI_mode)
        {
            if (gBrushedObj.DRVA_SEL)                      // Driver A selected
            {
                P3DIR &= ~nSCS_A;                          // nSCS_A set to HiZ
                P2OUT &= ~nSCS_B;                          // SPI nSCS_B low
            }
            else if (gBrushedObj.DRVB_SEL)                 // Driver B selected
            {
                P3DIR &= ~nSCS_A;                          // nSCS_A set to HiZ
                P2OUT &= ~nSCS_B;                          // SPI nSCS_B low
            }
        }

        while (!(IFG2 & UCB0TXIFG));                       // USCI_B0 TX buffer ready?
        UCB0TXBUF = reg_address;                           // Transmit first Byte, address Byte

        while (!(IFG2 & UCB0TXIFG));                       // USCI_B0 TX buffer ready?
        UCB0TXBUF = (uint8_t) ((reg_value >> 8) & 0xFF);   // Transmit Second Byte, upper data Byte

        while (!(IFG2 & UCB0TXIFG));                       // USCI_B0 TX buffer ready?
        UCB0TXBUF = (uint8_t) (reg_value & 0xFF);          // Transmit Third Byte, CRC Byte

        while (UCB0STAT & SPI_BUSY_FLAG);                  // Wait till Transmission is complete

        if (!gBrushedObj.drv_SPI_mode)
        {
            if (gBrushedObj.DRVA_SEL)                      // Driver A selected
            {
                P3OUT |= nSCS_A;                           // SPI nSCS_A high
            }
            else if (gBrushedObj.DRVB_SEL)                 // Driver B selected
            {
                P2OUT |= nSCS_B;                           // SPI nSCS_B high
            }
        }
        else if (gBrushedObj.drv_SPI_mode)
        {
            if (gBrushedObj.DRVA_SEL)                      // Driver A selected
            {
                P2OUT |= nSCS_B;                           // SPI nSCS_B high
                P3OUT |= nSCS_A;                           // SPI nSCS_A high
                P3DIR |= nSCS_A;                           // nSCS_A gpio output
            }
            else if (gBrushedObj.DRVB_SEL)                 // Driver B selected
            {
                P2OUT |= nSCS_B;                           // SPI nSCS_B high
                P3OUT |= nSCS_A;                           // SPI nSCS_A high
                P3DIR |= nSCS_A;                           // nSCS_A gpio output
            }
        }

        _delay_cycles(10);                                 // 625 ns settling delay
    }

    if((address == SPI_REG_CONFIG0) || (address == SPI_REG_CONFIG1) || (address == SPI_REG_CONFIG2) ||
       (address == SPI_REG_CONFIG3) || (address == SPI_REG_CONFIG4)) // CONFIGx registers
    {
        reg_value = data;
        crc_config = CRC_CONFIG();

        // write to registers of the selected device
        if (!gBrushedObj.drv_SPI_mode)
        {
            if (gBrushedObj.DRVA_SEL)                      // Driver A selected
            {
                P3OUT &= ~nSCS_A;                          // SPI nSCS_A low
            }
            else if (gBrushedObj.DRVB_SEL)                 // Driver B selected
            {
                P2OUT &= ~nSCS_B;                          // SPI nSCS_B low
            }
        }
        else if (gBrushedObj.drv_SPI_mode)
        {
            if (gBrushedObj.DRVA_SEL)                      // Driver A selected
            {
                P3DIR &= ~nSCS_A;                          // nSCS_A set to HiZ
                P2OUT &= ~nSCS_B;                          // SPI nSCS_B low
            }
            else if (gBrushedObj.DRVB_SEL)                 // Driver B selected
            {
                P3DIR &= ~nSCS_A;                          // nSCS_A set to HiZ
                P2OUT &= ~nSCS_B;                          // SPI nSCS_B low
            }
        }

        while (!(IFG2 & UCB0TXIFG));                       // USCI_B0 TX buffer ready?
        UCB0TXBUF = reg_address;                           // Transmit first Byte, address Byte

        while (!(IFG2 & UCB0TXIFG));                       // USCI_B0 TX buffer ready?
        UCB0TXBUF = (uint8_t) ((reg_value >> 8) & 0xFF);   // Transmit Second Byte, upper data Byte

        while (!(IFG2 & UCB0TXIFG));                       // USCI_B0 TX buffer ready?

        if (address != SPI_REG_CONFIG4)
            UCB0TXBUF = (uint8_t) (reg_value & 0xFF);      // Transmit Third Byte, lower data Byte
        else if (address == SPI_REG_CONFIG4)
            UCB0TXBUF = crc_config;                        // Transmit Third Byte, CRC Byte

        while (UCB0STAT & SPI_BUSY_FLAG);                  // Wait till Transmission is complete

        if (!gBrushedObj.drv_SPI_mode)
        {
            if (gBrushedObj.DRVA_SEL)                      // Driver A selected
            {
                P3OUT |= nSCS_A;                           // SPI nSCS_A high
            }
            else if (gBrushedObj.DRVB_SEL)                 // Driver B selected
            {
                P2OUT |= nSCS_B;                           // SPI nSCS_B high
            }
        }
        else if (gBrushedObj.drv_SPI_mode)
        {
            if (gBrushedObj.DRVA_SEL)                      // Driver A selected
            {
                P2OUT |= nSCS_B;                           // SPI nSCS_B high
                P3OUT |= nSCS_A;                           // SPI nSCS_A high
                P3DIR |= nSCS_A;                           // nSCS_A gpio output
            }
            else if (gBrushedObj.DRVB_SEL)                 // Driver B selected
            {
                P2OUT |= nSCS_B;                           // SPI nSCS_B high
                P3OUT |= nSCS_A;                           // SPI nSCS_A high
                P3DIR |= nSCS_A;                           // nSCS_A gpio output
            }
        }

        _delay_cycles(10);                                 // 625 ns settling delay

        if (address != SPI_REG_CONFIG4)
        {
            // write crc value to the lower byte of CONFIG4
            reg_address &= ~((address << SPI_ADDRESS_POS) & SPI_ADDRESS_MASK); // Clear address bits
            reg_address |= ((SPI_REG_CONFIG4 << SPI_ADDRESS_POS) & SPI_ADDRESS_MASK); // Set address to CONFIG4

            // write to selected device registers
            if (!gBrushedObj.drv_SPI_mode)
            {
                if (gBrushedObj.DRVA_SEL)                  // Driver A selected
                {
                    P3OUT &= ~nSCS_A;                      // SPI nSCS_A low
                }
                else if (gBrushedObj.DRVB_SEL)             // Driver B selected
                {
                    P2OUT &= ~nSCS_B;                      // SPI nSCS_B low
                }
            }
            else if (gBrushedObj.drv_SPI_mode)
            {
                if (gBrushedObj.DRVA_SEL)                  // Driver A selected
                {
                    P3DIR &= ~nSCS_A;                      // nSCS_A set to HiZ
                    P2OUT &= ~nSCS_B;                      // SPI nSCS_B low
                }
                else if (gBrushedObj.DRVB_SEL)             // Driver B selected
                {
                    P3DIR &= ~nSCS_A;                      // nSCS_A set to HiZ
                    P2OUT &= ~nSCS_B;                      // SPI nSCS_B low
                }
            }

            while (!(IFG2 & UCB0TXIFG));                   // USCI_B0 TX buffer ready?
            UCB0TXBUF = reg_address;                       // Transmit first Byte, address Byte

            while (!(IFG2 & UCB0TXIFG));                   // USCI_B0 TX buffer ready?
            UCB0TXBUF = configx[8];                        // Transmit Second Byte, upper data Byte (SPARE)

            while (!(IFG2 & UCB0TXIFG));                   // USCI_B0 TX buffer ready?
            UCB0TXBUF = crc_config;                        // Transmit Third Byte, CRC Byte

            while (UCB0STAT & SPI_BUSY_FLAG);              // Wait till Transmission is complete

            if (!gBrushedObj.drv_SPI_mode)
            {
                if (gBrushedObj.DRVA_SEL)                  // Driver A selected
                {
                    P3OUT |= nSCS_A;                       // SPI nSCS_A high
                }
                else if (gBrushedObj.DRVB_SEL)             // Driver B selected
                {
                    P2OUT |= nSCS_B;                       // SPI nSCS_B high
                }
            }
            else if (gBrushedObj.drv_SPI_mode)
            {
                if (gBrushedObj.DRVA_SEL)                  // Driver A selected
                {
                    P2OUT |= nSCS_B;                       // SPI nSCS_B high
                    P3OUT |= nSCS_A;                       // SPI nSCS_A high
                    P3DIR |= nSCS_A;                       // nSCS_A gpio output
                }
                else if (gBrushedObj.DRVB_SEL)             // Driver B selected
                {
                    P2OUT |= nSCS_B;                       // SPI nSCS_B high
                    P3OUT |= nSCS_A;                       // SPI nSCS_A high
                    P3DIR |= nSCS_A;                       // nSCS_A gpio output
                }
            }

            _delay_cycles(10);                             // 625 ns settling delay
        }
    }
    return 0;
} // end spi_writeRegister()
